from django.contrib.auth import authenticate, logout
from django.contrib.auth.decorators import login_required
from django.shortcuts import redirect, render
from usermerge.auth import login
from usermerge.forms import LoginForm
from usermerge.models import Platform

# Create your views here.
# https://docs.djangoproject.com/en/2.0/topics/http/views/

# For more information on authenticate() and logout() functions and login_required() decorator, see https://docs.djangoproject.com/en/2.0/topics/auth/default/ . For more information on login() function, see auth.py .
# For more information on working with forms, see https://docs.djangoproject.com/en/2.0/topics/forms/ .

def get_names_of_SoftLab_provided_platforms_from_DB():
    """
    Return the list of all the platform names, except for ECE-NTUA (this 'platform' is not provided by SoftLab),
    that exist in usermergeDB (see populateDB.py). This list can be used as part of the context
    while rendering our application pages, e.g. in the platform drop-down list of the login form.
    """
    platform_names = list(Platform.objects.order_by('id').values_list('name', flat = True))
    platform_names.remove('ECE-NTUA')
    return platform_names

def show_default_login_page(request):
    """
    Show login page without any validation/authentication error messages (that is, before submitting any invalid credentials).
    """
    return render(request, 'login.html', {'platform_list': get_names_of_SoftLab_provided_platforms_from_DB()})

def log_in(request):
    """
    Collect the credentials POSTed via the login form and validate them. If they are not valid, redirect the user to
    the login page and display the appropriate validation error messages. Otherwise, authenticate them. If they do
    not correspond to any User or Admin instance of usermergeDB (see models.py), redirect the user to the login page and
    display the appropriate authentication error message. Otherwise, log the user in and redirect him/her to the
    appropriate home page depending on his/her representative model/class (User/Admin from usermerge.models).
    """
    form = LoginForm(request.POST)
    if form.is_valid():
        platform = form.cleaned_data['platform']
        username = form.cleaned_data['username']
        password = form.cleaned_data['password']
        user = authenticate(request, platform = platform, username = username, password = password)
        if user:
            login(request, user)
            # User instances have an ece_id attribute, whereas Admin instances do not.
            # Therefore, if user has an ece_id attribute, he/she is a User. Otherwise, he/she is an Admin.
            if hasattr(user, 'ece_id'):
                return redirect('user_home')
            else:
                return redirect('admin_home')
        else:
            return render(request, 'login.html',
                          {'platform_list': get_names_of_SoftLab_provided_platforms_from_DB(),
                           'error_messages': ['Τα δηλωθέντα διαπιστευτήρια δεν ανταποκρίνονται σε κανένα χρήστη της βάσης!\n'
                                              'Παρακαλούμε ελέγξτε την ορθότητα των διαπιστευτηρίων και δοκιμάστε ξανά!']})
    else:
        return render(request, 'login.html',
                      {'platform_list': get_names_of_SoftLab_provided_platforms_from_DB(),
                       # https://docs.python.org/3/tutorial/datastructures.html#list-comprehensions
                       'error_messages': [list(form.errors.values())[k-1][0] for k in range(len(form.errors.values()))]})

@login_required(redirect_field_name = None)
def show_user_home_page(request):
    return render(request, 'user_home.html', {})

@login_required(redirect_field_name = None)
def show_admin_home_page(request):
    return render(request, 'admin_home.html', {})

@login_required(redirect_field_name = None)
def log_out(request):
    """
    Log the authenticated user out and redirect him/her to the logout page.
    """
    logout(request)
    return render(request, 'logout.html', {})

